@@ -45,20 +45,20 @@ gchar *xfercmd = NULL;
 alpm_list_t *holdpkgs = NULL;
 alpm_list_t *syncfirsts = NULL;
 
-gchar *
+const gchar *
 pk_backend_get_description (PkBackend *self)
 {
 	g_return_val_if_fail (self != NULL, NULL);
 
-	return g_strdup ("alpm");
+	return "alpm";
 }
 
-gchar *
+const gchar *
 pk_backend_get_author (PkBackend *self)
 {
 	g_return_val_if_fail (self != NULL, NULL);
 
-	return g_strdup ("Jonathan Conder <jonno.conder@gmail.com>");
+	return "Jonathan Conder <jonno.conder@gmail.com>";
 }
 
 static gboolean
@@ -284,9 +284,9 @@ pk_backend_initialize_alpm (PkBackend *self, GError **error)
 	backend = self;
 	alpm_option_set_logcb (alpm, pk_backend_logcb);
 
-	localdb = alpm_option_get_localdb (alpm);
+	localdb = alpm_get_localdb (alpm);
 	if (localdb == NULL) {
-		enum _alpm_errno_t errno = alpm_errno (alpm);
+		alpm_errno_t errno = alpm_errno (alpm);
 		g_set_error (error, ALPM_ERROR, errno, "[%s]: %s", "local",
 			     alpm_strerror (errno));
 	}
@@ -348,14 +348,15 @@ pk_backend_get_filters (PkBackend *self)
 	return pk_bitfield_from_enums (PK_FILTER_ENUM_INSTALLED, -1);
 }
 
-gchar *
+gchar **
 pk_backend_get_mime_types (PkBackend *self)
 {
-	g_return_val_if_fail (self != NULL, NULL);
-
 	/* packages currently use .pkg.tar.gz and .pkg.tar.xz */
-	return g_strdup ("application/x-compressed-tar;"
-			 "application/x-xz-compressed-tar");
+	const gchar *mime_types[] = {
+				"application/x-compressed-tar",
+				"application/x-xz-compressed-tar",
+				NULL };
+	return g_strdupv ((gchar **) mime_types);
 }
 
 void
@@ -374,10 +375,10 @@ pk_backend_run (PkBackend *self, PkStatusEnum status, PkBackendThreadFunc func)
 
 	g_static_mutex_unlock (&mutex);
 
-	pk_backend_set_allow_cancel (self, TRUE);
+	pk_backend_job_set_allow_cancel (self, TRUE);
 
-	pk_backend_set_status (self, status);
-	pk_backend_thread_create (self, func);
+	pk_backend_job_set_status (self, status);
+	pk_backend_job_thread_create (self, func);
 }
 
 void
@@ -418,7 +419,7 @@ pk_backend_finish (PkBackend *self, GError *error)
 
 	g_return_val_if_fail (self != NULL, FALSE);
 
-	pk_backend_set_allow_cancel (self, FALSE);
+	pk_backend_job_set_allow_cancel (self, FALSE);
 
 	g_static_mutex_lock (&mutex);
 
@@ -436,10 +437,10 @@ pk_backend_finish (PkBackend *self, GError *error)
 	}
 
 	if (cancelled) {
-		pk_backend_set_status (self, PK_STATUS_ENUM_CANCEL);
+		pk_backend_job_set_status (self, PK_STATUS_ENUM_CANCEL);
 	}
 
-	pk_backend_thread_finished (self);
+	pk_backend_job_finished (self);
 	return (error == NULL);
 }
 
