@@ -14,7 +14,7 @@ command_not_found_handle () {
 	[ ! -S /var/run/dbus/system_bus_socket ] && runcnf=0
 
 	# don't run if packagekitd doesn't exist in the _system_ root
-	[ ! -x /usr/libexec/packagekitd ] && runcnf=0
+	[ ! -x /home/hughsie/.root/libexec/packagekitd ] && runcnf=0
 
 	# run the command, or just print a warning
 	if [ $runcnf -eq 1 ]; then
