@@ -51,6 +51,7 @@ static void     pk_package_sack_finalize	(GObject     *object);
  **/
 struct _PkPackageSackPrivate
 {
+	GHashTable		*table;
 	GPtrArray		*array;
 	PkClient		*client;
 };
@@ -74,7 +75,9 @@ void
 pk_package_sack_clear (PkPackageSack *sack)
 {
 	g_return_if_fail (PK_IS_PACKAGE_SACK (sack));
+
 	g_ptr_array_set_size (sack->priv->array, 0);
+	g_hash_table_remove_all (sack->priv->table);
 }
 
 /**
@@ -117,7 +120,7 @@ pk_package_sack_get_ids (PkPackageSack *sack)
 
 	array = sack->priv->array;
 	package_ids = g_new0 (gchar *, array->len + 1);
-	for (i=0; i<array->len; i++) {
+	for (i = 0; i < array->len; i++) {
 		package = g_ptr_array_index (array, i);
 		package_ids[i] = g_strdup (pk_package_get_id (package));
 	}
@@ -232,7 +235,11 @@ pk_package_sack_add_package (PkPackageSack *sack, PkPackage *package)
 	g_return_val_if_fail (PK_IS_PACKAGE (package), FALSE);
 
 	/* add to array */
-	g_ptr_array_add (sack->priv->array, g_object_ref (package));
+	g_ptr_array_add (sack->priv->array,
+			 g_object_ref (package));
+	g_hash_table_insert (sack->priv->table,
+			     g_strdup (pk_package_get_id (package)),
+			     g_object_ref (package));
 
 	return TRUE;
 }
@@ -250,7 +257,9 @@ pk_package_sack_add_package (PkPackageSack *sack, PkPackage *package)
  * Since: 0.5.2
  **/
 gboolean
-pk_package_sack_add_package_by_id (PkPackageSack *sack, const gchar *package_id, GError **error)
+pk_package_sack_add_package_by_id (PkPackageSack *sack,
+				   const gchar *package_id,
+				   GError **error)
 {
 	PkPackage *package;
 	gboolean ret;
@@ -276,7 +285,9 @@ out:
  * pk_package_sack_add_packages_from_line:
  **/
 static void
-pk_package_sack_add_packages_from_line (PkPackageSack *sack, const gchar *package_str, GError **error)
+pk_package_sack_add_packages_from_line (PkPackageSack *sack,
+					const gchar *package_str,
+					GError **error)
 {
 	GError *error_local = NULL;
 	gboolean ret;
@@ -314,7 +325,7 @@ out:
 }
 
 /**
- * pk_package_sack_add_packages_from_file
+ * pk_package_sack_add_packages_from_file:
  * @sack: a valid #PkPackageSack instance
  * @file: a valid package-list file
  * @error: a %GError to put the error code and message in, or %NULL
@@ -325,7 +336,9 @@ out:
  *
  **/
 gboolean
-pk_package_sack_add_packages_from_file (PkPackageSack *sack, GFile *file, GError **error)
+pk_package_sack_add_packages_from_file (PkPackageSack *sack,
+					GFile *file,
+					GError **error)
 {
 	GError *error_local = NULL;
 	gboolean ret = TRUE;
@@ -367,6 +380,51 @@ pk_package_sack_add_packages_from_file (PkPackageSack *sack, GFile *file, GError
 	return ret;
 }
 
+/**
+ * pk_package_sack_add_packages_from_file:
+ * @sack: a valid #PkPackageSack instance
+ * @file: a valid package-list file
+ * @error: a %GError to put the error code and message in, or %NULL
+ *
+ * Write the contents of a PkPackageSack to a package-list file.
+ *
+ * Return value: %TRUE if there were no errors.
+ *
+ * Since: 0.8.6
+ **/
+gboolean
+pk_package_sack_to_file (PkPackageSack *sack, GFile *file, GError **error)
+{
+	gboolean ret;
+	GString *string;
+	guint i;
+	PkPackage *pkg;
+
+	string = g_string_new ("");
+	for (i = 0; i < sack->priv->array->len; i++) {
+		pkg = g_ptr_array_index (sack->priv->array, i);
+		g_string_append_printf (string,
+					"%s\t%s\t%s\n",
+					pk_info_enum_to_string (pk_package_get_info (pkg)),
+					pk_package_get_id (pkg),
+					pk_package_get_summary (pkg));
+	}
+	ret = g_file_replace_contents (file,
+				       string->str,
+				       string->len,
+				       NULL,
+				       FALSE,
+				       G_FILE_CREATE_NONE,
+				       NULL,
+				       NULL,
+				       error);
+	if (!ret)
+		goto out;
+out:
+	g_string_free (string, FALSE);
+	return ret;
+}
+
 /**
  * pk_package_sack_remove_package:
  * @sack: a valid #PkPackageSack instance
@@ -387,6 +445,7 @@ pk_package_sack_remove_package (PkPackageSack *sack, PkPackage *package)
 	g_return_val_if_fail (PK_IS_PACKAGE (package), FALSE);
 
 	/* remove from array */
+	g_hash_table_remove (sack->priv->table, pk_package_get_id (package));
 	ret = g_ptr_array_remove (sack->priv->array, package);
 
 	return ret;
@@ -405,10 +464,10 @@ pk_package_sack_remove_package (PkPackageSack *sack, PkPackage *package)
  * Since: 0.5.2
  **/
 gboolean
-pk_package_sack_remove_package_by_id (PkPackageSack *sack, const gchar *package_id)
+pk_package_sack_remove_package_by_id (PkPackageSack *sack,
+				      const gchar *package_id)
 {
 	PkPackage *package;
-	const gchar *id;
 	gboolean ret = FALSE;
 	guint i;
 	GPtrArray *array;
@@ -417,11 +476,10 @@ pk_package_sack_remove_package_by_id (PkPackageSack *sack, const gchar *package_
 	g_return_val_if_fail (package_id != NULL, FALSE);
 
 	array = sack->priv->array;
-	for (i=0; i<array->len; i++) {
+	for (i = 0; i < array->len; i++) {
 		package = g_ptr_array_index (array, i);
-		id = pk_package_get_id (package);
-		if (g_strcmp0 (package_id, id) == 0) {
-			g_ptr_array_remove_index (array, i);
+		if (g_strcmp0 (package_id, pk_package_get_id (package)) == 0) {
+			pk_package_sack_remove_package (sack, package);
 			ret = TRUE;
 			break;
 		}
@@ -444,7 +502,9 @@ pk_package_sack_remove_package_by_id (PkPackageSack *sack, const gchar *package_
  * Since: 0.6.3
  **/
 gboolean
-pk_package_sack_remove_by_filter (PkPackageSack *sack, PkPackageSackFilterFunc filter_cb, gpointer user_data)
+pk_package_sack_remove_by_filter (PkPackageSack *sack,
+				  PkPackageSackFilterFunc filter_cb,
+				  gpointer user_data)
 {
 	gboolean ret = FALSE;
 	PkPackage *package;
@@ -483,24 +543,14 @@ pk_package_sack_remove_by_filter (PkPackageSack *sack, PkPackageSackFilterFunc f
 PkPackage *
 pk_package_sack_find_by_id (PkPackageSack *sack, const gchar *package_id)
 {
-	PkPackage *package_tmp;
-	const gchar *id;
 	PkPackage *package = NULL;
-	guint i;
-	guint len;
 
 	g_return_val_if_fail (PK_IS_PACKAGE_SACK (sack), NULL);
 	g_return_val_if_fail (package_id != NULL, NULL);
 
-	len = sack->priv->array->len;
-	for (i=0; i<len; i++) {
-		package_tmp = g_ptr_array_index (sack->priv->array, i);
-		id = pk_package_get_id (package_tmp);
-		if (g_strcmp0 (package_id, id) == 0) {
-			package = g_object_ref (package_tmp);
-			break;
-		}
-	}
+	package = g_hash_table_lookup (sack->priv->table, package_id);
+	if (package != NULL)
+		g_object_ref (package);
 
 	return package;
 }
@@ -615,7 +665,7 @@ pk_package_sack_get_total_bytes (PkPackageSack *sack)
 	g_return_val_if_fail (PK_IS_PACKAGE_SACK (sack), FALSE);
 
 	array = sack->priv->array;
-	for (i=0; i<array->len; i++) {
+	for (i = 0; i < array->len; i++) {
 		package = g_ptr_array_index (array, i);
 		g_object_get (package,
 			      "size", &bytes_tmp,
@@ -641,7 +691,7 @@ pk_package_sack_get_package_ids (PkPackageSack *sack)
 	/* create array of package_ids */
 	array = sack->priv->array;
 	package_ids = g_new0 (gchar *, array->len+1);
-	for (i=0; i<array->len; i++) {
+	for (i = 0; i < array->len; i++) {
 		package = g_ptr_array_index (array, i);
 		id = pk_package_get_id (package);
 		package_ids[i] = g_strdup (id);
@@ -720,7 +770,7 @@ pk_package_sack_resolve_cb (GObject *source_object, GAsyncResult *res, PkPackage
 	}
 
 	/* set data on each item */
-	for (i=0; i<packages->len; i++) {
+	for (i = 0; i < packages->len; i++) {
 		item = g_ptr_array_index (packages, i);
 		g_object_get (item,
 			      "info", &info,
@@ -873,7 +923,7 @@ pk_package_sack_get_details_cb (GObject *source_object, GAsyncResult *res, PkPac
 	}
 
 	/* set data on each item */
-	for (i=0; i<details->len; i++) {
+	for (i = 0; i < details->len; i++) {
 		item = g_ptr_array_index (details, i);
 
 		g_object_get (item,
@@ -982,9 +1032,9 @@ pk_package_sack_get_update_detail_cb (GObject *source_object, GAsyncResult *res,
 	gchar *package_id;
 	gchar *updates;
 	gchar *obsoletes;
-	gchar *vendor_url;
-	gchar *bugzilla_url;
-	gchar *cve_url;
+	gchar **vendor_urls;
+	gchar **bugzilla_urls;
+	gchar **cve_urls;
 	PkRestartEnum restart;
 	gchar *update_text;
 	gchar *changelog;
@@ -1011,15 +1061,15 @@ pk_package_sack_get_update_detail_cb (GObject *source_object, GAsyncResult *res,
 	}
 
 	/* set data on each item */
-	for (i=0; i<update_details->len; i++) {
+	for (i = 0; i < update_details->len; i++) {
 		item = g_ptr_array_index (update_details, i);
 		g_object_get (item,
 			      "package-id", &package_id,
 			      "updates", &updates,
 			      "obsoletes", &obsoletes,
-			      "vendor-url", &vendor_url,
-			      "bugzilla-url", &bugzilla_url,
-			      "cve-url", &cve_url,
+			      "vendor-urls", &vendor_urls,
+			      "bugzilla-urls", &bugzilla_urls,
+			      "cve-urls", &cve_urls,
 			      "restart", &restart,
 			      "update-text", &update_text,
 			      "changelog", &changelog,
@@ -1039,9 +1089,9 @@ pk_package_sack_get_update_detail_cb (GObject *source_object, GAsyncResult *res,
 		g_object_set (package,
 			      "update-updates", updates,
 			      "update-obsoletes", obsoletes,
-			      "update-vendor-url", vendor_url,
-			      "update-bugzilla-url", bugzilla_url,
-			      "update-cve-url", cve_url,
+			      "update-vendor-urls", vendor_urls,
+			      "update-bugzilla-urls", bugzilla_urls,
+			      "update-cve-urls", cve_urls,
 			      "update-restart", restart,
 			      "update-text", update_text,
 			      "update-changelog", changelog,
@@ -1054,9 +1104,9 @@ skip:
 		g_free (package_id);
 		g_free (updates);
 		g_free (obsoletes);
-		g_free (vendor_url);
-		g_free (bugzilla_url);
-		g_free (cve_url);
+		g_strfreev (vendor_urls);
+		g_strfreev (bugzilla_urls);
+		g_strfreev (cve_urls);
 		g_free (update_text);
 		g_free (changelog);
 		g_free (issued);
@@ -1160,6 +1210,7 @@ pk_package_sack_init (PkPackageSack *sack)
 	sack->priv = PK_PACKAGE_SACK_GET_PRIVATE (sack);
 	priv = sack->priv;
 
+	priv->table = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
 	priv->array = g_ptr_array_new_with_free_func (g_object_unref);
 	priv->client = pk_client_new ();
 }
@@ -1174,6 +1225,7 @@ pk_package_sack_finalize (GObject *object)
 	PkPackageSackPrivate *priv = sack->priv;
 
 	g_ptr_array_unref (priv->array);
+	g_hash_table_unref (priv->table);
 	g_object_unref (priv->client);
 
 	G_OBJECT_CLASS (pk_package_sack_parent_class)->finalize (object);
