@@ -192,10 +192,10 @@ pk_service_pack_extract (const gchar *filename, const gchar *directory, GError *
 	struct archive_entry *entry;
 	int r;
 	int retval;
+	gchar *buf;
 
 	/* save the PWD as we chdir to extract */
-	gchar *buf = getcwd (NULL, 0);
-
+	buf = getcwd (NULL, 0);
 	if (buf == NULL) {
 		g_set_error_literal (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_SETUP,
 				      "failed to get cwd");
@@ -207,7 +207,7 @@ pk_service_pack_extract (const gchar *filename, const gchar *directory, GError *
 	archive_read_support_format_tar (arch);
 
 	/* open the tar file */
-	r = archive_read_open_file (arch, filename, 10240);
+	r = archive_read_open_filename (arch, filename, 10240);
 	if (r) {
 		g_set_error (error, PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_EXTRACTION,
 				      "cannot open: %s", archive_error_string (arch));
@@ -245,14 +245,15 @@ pk_service_pack_extract (const gchar *filename, const gchar *directory, GError *
 out:
 	/* close the archive */
 	if (arch != NULL) {
-		archive_read_close (arch);
-		archive_read_finish (arch);
+		archive_read_free (arch);
 	}
 
 	/* switch back to PWD */
-	retval = chdir (buf);
-	if (retval != 0)
-		g_warning ("cannot chdir back!");
+	if (buf != NULL) {
+		retval = chdir (buf);
+		if (retval != 0)
+			g_warning ("cannot chdir back!");
+	}
 	free (buf);
 
 	return ret;
@@ -301,8 +302,9 @@ static gchar *
 pk_service_pack_create_temporary_directory (const gchar *prefix)
 {
 	gboolean ret;
-	gchar *random_str;
 	gchar *directory = NULL;
+	gchar *random_str;
+	gint rc;
 
 	/* ensure path does not already exist */
 	do {
@@ -319,7 +321,11 @@ pk_service_pack_create_temporary_directory (const gchar *prefix)
 	} while (ret);
 
 	/* create so only user (root) has rwx access */
-	g_mkdir (directory, 0700);
+	rc = g_mkdir (directory, 0700);
+	if (rc < 0) {
+		g_free (directory);
+		directory = NULL;
+	}
 
 	return directory;
 }
@@ -572,7 +578,11 @@ pk_service_pack_create_from_files (PkServicePackState *state, gchar **file_array
 
 	/* we can only write tar achives */
 	arch = archive_write_new ();
+#if ARCHIVE_VERSION_NUMBER >= 3000000
+	archive_write_add_filter_none (arch);
+#else
 	archive_write_set_compression_none (arch);
+#endif
 	archive_write_set_format_ustar (arch);
 	archive_write_open_filename (arch, state->filename);
 
@@ -595,8 +605,7 @@ out:
 
 	/* close the archive */
 	if (arch != NULL) {
-		archive_write_close (arch);
-		archive_write_finish (arch);
+		archive_write_free (arch);
 	}
 	return ret;
 }
