@@ -1254,6 +1254,187 @@ out:
 	return retval;
 }
 
+/**
+ * pk_engine_package_name_in_strv:
+ **/
+static gboolean
+pk_engine_package_name_in_strv (gchar **strv, PkPackage *pkg)
+{
+	guint i;
+	for (i = 0; strv[i] != NULL; i++) {
+		if (g_strcmp0 (strv[i], pk_package_get_name (pkg)) == 0)
+			return TRUE;
+	}
+	return FALSE;
+}
+
+/**
+ * pk_engine_get_package_history_pkg:
+ *
+ * Create a 'a{sv}' GVariant instance from all the PkTransactionPast data
+ **/
+static GVariant *
+pk_engine_get_package_history_pkg (PkTransactionPast *item, PkPackage *pkg)
+{
+	GVariantBuilder builder;
+	g_variant_builder_init (&builder, G_VARIANT_TYPE_ARRAY);
+	g_variant_builder_add (&builder, "{sv}", "info",
+			       g_variant_new_uint32 (pk_package_get_info (pkg)));
+	g_variant_builder_add (&builder, "{sv}", "source",
+			       g_variant_new_string (pk_package_get_data (pkg)));
+	g_variant_builder_add (&builder, "{sv}", "version",
+			       g_variant_new_string (pk_package_get_version (pkg)));
+	g_variant_builder_add (&builder, "{sv}", "timestamp",
+			       g_variant_new_uint64 (pk_transaction_past_get_timestamp (item)));
+	g_variant_builder_add (&builder, "{sv}", "user-id",
+			       g_variant_new_uint32 (pk_transaction_past_get_uid (item)));
+	return g_variant_builder_end (&builder);
+}
+
+/**
+ * pk_engine_is_package_history_interesing:
+ **/
+static gboolean
+pk_engine_is_package_history_interesing (PkPackage *package)
+{
+	gboolean ret;
+
+	switch (pk_package_get_info (package)) {
+	case PK_INFO_ENUM_INSTALLING:
+	case PK_INFO_ENUM_REMOVING:
+	case PK_INFO_ENUM_UPDATING:
+		ret = TRUE;
+		break;
+	default:
+		ret = FALSE;
+		break;
+	}
+	return ret;
+}
+
+/**
+ * pk_engine_get_package_history:
+ **/
+static GVariant *
+pk_engine_get_package_history (PkEngine *engine,
+			       gchar **package_names,
+			       guint max_size,
+			       GError **error)
+{
+	const gchar *data;
+	const gchar *pkgname;
+	gboolean ret;
+	gchar *key;
+	gchar **package_lines;
+	GHashTable *deduplicate_hash;
+	GHashTable *pkgname_hash;
+	gint64 timestamp;
+	GList *keys = NULL;
+	GList *l;
+	GList *list;
+	GPtrArray *array = NULL;
+	guint i;
+	GVariantBuilder builder;
+	GVariant *value = NULL;
+	PkPackage *package_tmp;
+	PkTransactionPast *item;
+
+	list = pk_transaction_db_get_list (engine->priv->transaction_db, max_size);
+
+	/* simplify the loop */
+	if (max_size == 0)
+		max_size = G_MAXUINT;
+
+	pkgname_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) g_ptr_array_unref);
+	deduplicate_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	package_tmp = pk_package_new ();
+	for (l = list; l != NULL; l = l->next) {
+		item = PK_TRANSACTION_PAST (l->data);
+
+		/* ignore anything that failed */
+		if (!pk_transaction_past_get_succeeded (item))
+			continue;
+
+		/* split up data */
+		data = pk_transaction_past_get_data (item);
+		if (data == NULL)
+			continue;
+		package_lines = g_strsplit (data, "\n", -1);
+		for (i = 0; package_lines[i] != NULL; i++) {
+			ret = pk_package_parse (package_tmp, package_lines[i], error);
+			g_assert (ret);
+
+			/* not the package we care about */
+			if (!pk_engine_package_name_in_strv (package_names, package_tmp))
+				continue;
+
+			/* not a state we care about */
+			if (!pk_engine_is_package_history_interesing (package_tmp))
+				continue;
+
+			/* transactions without a timestamp are not interesting */
+			timestamp = pk_transaction_past_get_timestamp (item);
+			if (timestamp == 0)
+				continue;
+
+			/* de-duplicate the entry, in the case of multiarch */
+			key = g_strdup_printf ("%s-%" G_GINT64_FORMAT,
+					       pk_package_get_name (package_tmp),
+					       timestamp);
+			if (g_hash_table_lookup (deduplicate_hash, key) != NULL) {
+				g_free (key);
+				continue;
+			}
+			g_hash_table_insert (deduplicate_hash, key, package_lines[i]);
+
+			/* get the blob for this data item */
+			value = pk_engine_get_package_history_pkg (item, package_tmp);
+			if (value == NULL)
+				continue;
+
+			/* find the array */
+			pkgname = pk_package_get_name (package_tmp);
+			array = g_hash_table_lookup (pkgname_hash, pkgname);
+			if (array == NULL) {
+				array = g_ptr_array_new ();
+				g_hash_table_insert (pkgname_hash,
+						     g_strdup (pkgname),
+						     array);
+			}
+			g_ptr_array_add (array, value);
+		}
+		g_strfreev (package_lines);
+	}
+
+	/* no history returns an empty array */
+	if (g_hash_table_size (pkgname_hash) == 0) {
+		value = g_variant_new_array (G_VARIANT_TYPE ("{saa{sv}}"), NULL, 0);
+		goto out;
+	}
+
+	/* we have a hash of pkgname:GPtrArray where the GPtrArray is an array
+	 * of GVariants of type a{sv} */
+	g_variant_builder_init (&builder, G_VARIANT_TYPE_ARRAY);
+	keys = g_hash_table_get_keys (pkgname_hash);
+	for (l = keys; l != NULL; l = l->next) {
+		pkgname = l->data;
+		array = g_hash_table_lookup (pkgname_hash, pkgname);
+		/* create aa{sv} */
+		value = g_variant_new_array (NULL,
+					     (GVariant * const *) array->pdata,
+					     MIN (array->len, max_size));
+		g_variant_builder_add (&builder, "{s@aa{sv}}", pkgname, value);
+	}
+	value = g_variant_builder_end (&builder);
+out:
+	g_list_free (keys);
+	g_hash_table_unref (pkgname_hash);
+	g_hash_table_unref (deduplicate_hash);
+	g_object_unref (package_tmp);
+	g_list_free_full (list, (GDestroyNotify) g_object_unref);
+	return value;
+}
+
 /**
  * pk_engine_daemon_method_call:
  **/
@@ -1269,11 +1450,13 @@ pk_engine_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 	GError *error = NULL;
 	guint time_since;
 	GVariant *value = NULL;
+	GVariant *tuple = NULL;
 	PkAuthorizeEnum result_enum;
 	PkEngine *engine = PK_ENGINE (user_data);
 	PkRoleEnum role;
 	gchar **transaction_list;
 	gchar **array = NULL;
+	gchar **package_names;
 	guint size;
 	gboolean is_priority = TRUE;
 
@@ -1298,6 +1481,31 @@ pk_engine_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 		goto out;
 	}
 
+	if (g_strcmp0 (method_name, "GetPackageHistory") == 0) {
+		g_variant_get (parameters, "(^a&su)", &package_names, &size);
+		if (package_names == NULL || g_strv_length (package_names) == 0) {
+			g_dbus_method_invocation_return_error (invocation,
+							       PK_ENGINE_ERROR,
+							       PK_ENGINE_ERROR_NOT_SUPPORTED,
+							       "history for package name invalid");
+			goto out;
+		}
+		value = pk_engine_get_package_history (engine, package_names, size, &error);
+		if (value == NULL) {
+			g_dbus_method_invocation_return_error (invocation,
+							       PK_ENGINE_ERROR,
+							       PK_ENGINE_ERROR_NOT_SUPPORTED,
+							       "history for package name %s failed: %s",
+							       package_names[0],
+							       error->message);
+			g_error_free (error);
+			goto out;
+		}
+		tuple = g_variant_new_tuple (&value, 1);
+		g_dbus_method_invocation_return_value (invocation, tuple);
+		goto out;
+	}
+
 	if (g_strcmp0 (method_name, "CreateTransaction") == 0) {
 
 		g_debug ("CreateTransaction method called");
